<?php

declare(strict_types=1);

namespace AsyncAws\Core\Exception;

/**
 * All exceptions implements this interface.
 *
 * @author Tobias Nyholm <tobias.nyholm@gmail.com>
 */
interface Exception extends \Throwable
{
}
